/*
 * Copyright (c) 2005-, Claybird
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:

 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Claybird nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "stdafx.h"
#include "FileListView.h"
#include "../resource.h"
#include "../Dialogs/LogDialog.h"
#include "../Dialogs/LFFolderDialog.h"
#include "../ConfigCode/ConfigFileListWindow.h"
#include "../Utilities/StringUtil.h"
#include <sstream>

CFileListView::CFileListView(CConfigManager& rConfig,CFileListModel& rModel):
	mr_Config(rConfig),
	mr_Model(rModel),
	m_bDisplayFileSizeInByte(false),
	m_bPathOnly(false),
	//m_DnDSource(m_TempDirManager),
	m_DropTarget(this),
	m_nDropHilight(-1),
	m_hFrameWnd(NULL),
	m_TempDirMgr(_T("lhaf"))
{
}

LRESULT CFileListView::OnCreate(LPCREATESTRUCT lpcs)
{
	LRESULT lRes=DefWindowProc();
	//SetFont(AtlGetDefaultGuiFont());

	// C[WXg쐬
	m_ShellDataManager.Init();
	SetImageList(m_ShellDataManager.GetImageList(true),LVSIL_NORMAL);
	SetImageList(m_ShellDataManager.GetImageList(false),LVSIL_SMALL);

	//Jwb_̃\[gACR
	m_SortImageList.CreateFromImage(MAKEINTRESOURCE(IDB_BITMAP_SORTMARK),16,1,CLR_DEFAULT,IMAGE_BITMAP,LR_CREATEDIBSECTION);

	mr_Model.addEventListener(m_hWnd);
	return lRes;
}

LRESULT CFileListView::OnDestroy()
{
	mr_Model.removeEventListener(m_hWnd);
	return 0;
}



bool CFileListView::SetColumnState(const int* pColIdx2TypeID, const int *pFileInfoWidthArray)
{
	//̃J폜
	if(GetHeader().IsWindow()){
		int nCount=GetHeader().GetItemCount();
		for(;nCount>0;nCount--){
			DeleteColumn(nCount-1);
		}
	}

	memset(m_ColIdx2FileInfoType,-1,sizeof(m_ColIdx2FileInfoType));

//========================================
//      Xgr[ɃJǉ
//========================================
//Xgr[ɃJǉ邽߂̃}N
#define ADD_COLUMNITEM(x,widthDefault,pos) \
{if(-1!=UtilCheckNumberArray(pColIdx2TypeID, FILEINFO_ITEM_COUNT, FILEINFO_##x)){			\
	int width = widthDefault;																\
	if(pFileInfoWidthArray[FILEINFO_##x]>0)width = pFileInfoWidthArray[FILEINFO_##x];		\
	int nIndex = InsertColumn(FILEINFO_##x, CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_##x)), pos, width,-1);\
	if(nIndex < 0 || nIndex >= FILEINFO_ITEM_COUNT)return false;							\
	m_ColIdx2FileInfoType[nIndex] = FILEINFO_##x;											\
}}

	//t@C
	ADD_COLUMNITEM(FILENAME,100,LVCFMT_LEFT);
	//tpX
	ADD_COLUMNITEM(FULLPATH,200,LVCFMT_LEFT);
	//t@CTCY
	ADD_COLUMNITEM(ORIGINALSIZE,90,LVCFMT_RIGHT);
	//t@C
	ADD_COLUMNITEM(TYPENAME,120,LVCFMT_LEFT);
	//XV
	ADD_COLUMNITEM(FILETIME,120,LVCFMT_LEFT);
	//
	ADD_COLUMNITEM(ATTRIBUTE,60,LVCFMT_LEFT);
	//kTCY
	ADD_COLUMNITEM(COMPRESSEDSIZE,90,LVCFMT_RIGHT);
	//k\bh
	ADD_COLUMNITEM(METHOD,60,LVCFMT_LEFT);
	//k
	ADD_COLUMNITEM(RATIO,60,LVCFMT_RIGHT);
	//CRC
	ADD_COLUMNITEM(CRC,60,LVCFMT_LEFT);


	//----------------------
	// J̕яݒ
	//----------------------
	int Count=0;
	//LȃACe߂
	for(;Count<FILEINFO_ITEM_COUNT;Count++){
		if(-1==pColIdx2TypeID[Count])break;
	}

	int colIdx2TypeID_copy[FILEINFO_ITEM_COUNT];
	memcpy(colIdx2TypeID_copy, pColIdx2TypeID, Count*sizeof(int));
	SetColumnOrderArray(Count,colIdx2TypeID_copy);

	//Jwb_̃\[gACR
	if(GetHeader().IsWindow()){
		GetHeader().SetImageList(m_SortImageList);
	}
	UpdateSortIcon();
	return true;
}

void CFileListView::GetColumnState(int* pColIdx2TypeID, int *pFileInfoWidthArray)
{
	//J̕я擾
	const int nCount=GetHeader().GetItemCount();
	ASSERT(nCount<=FILEINFO_ITEM_COUNT);

	memset(pColIdx2TypeID,-1,sizeof(int)*FILEINFO_ITEM_COUNT);
	GetColumnOrderArray(nCount,pColIdx2TypeID);

	//J擾
	memset(pFileInfoWidthArray,-1,sizeof(int)*FILEINFO_ITEM_COUNT);
	for(int typeID=0;typeID<nCount;typeID++){
		pFileInfoWidthArray[typeID] = GetColumnWidth(pColIdx2TypeID[typeID]);
	}
}

void CFileListView::GetSelectedItems(std::list<ARCHIVE_ENTRY_INFO_TREE*> &items)
{
	items.clear();
	int nIndex=-1;
	for(;;){
		nIndex = GetNextItem(nIndex, LVNI_ALL | LVNI_SELECTED);
		if(-1==nIndex)break;
		ARCHIVE_ENTRY_INFO_TREE* lpNode=mr_Model.GetFileListItemByIndex(nIndex);

		ASSERT(lpNode);

		items.push_back(lpNode);
	}
}

LRESULT CFileListView::OnFileListNewContent(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	TRACE(__FUNCTIONW__ _T("\n"));

	DeleteAllItems();
	SetItemCount(0);
	if(mr_Model.IsOK()){
		ARCHIVE_ENTRY_INFO_TREE* lpCurrent=mr_Model.GetCurrentNode();
		ASSERT(lpCurrent);
		if(lpCurrent){
			SetItemCount(lpCurrent->GetNumChildren());
			SetItemState(0,LVIS_FOCUSED,LVIS_FOCUSED);
		}
	}

	//t@Chbv̎󂯓:󂯓ꋑۂDragOvers
	EnableDropTarget(true);
	return 0;
}

LRESULT CFileListView::OnFileListUpdated(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	TRACE(__FUNCTIONW__ _T("\n"));

	Invalidate();

	return 0;
}

LRESULT CFileListView::OnDblClick(LPNMHDR pnmh)
{
	TRACE(__FUNCTIONW__ _T("\n"));
	//----------------------------------------------------------------------
	//IACe̎:
	// I֘AtŊJ
	//ShiftĂ:
	// I֘AtŊJ
	//IACe̎:
	// tH__uNbN/Enterꍇɂ͂̃tH_J
	// It@Cꍇɂ͊֘AtŊJ
	//----------------------------------------------------------------------

	//IACe̎
	//ShiftĂA֘AtŊJ
	if(GetKeyState(VK_SHIFT)<0||GetSelectedCount()>=2){
		OpenAssociation(false,true);
		return 0;
	}

	//IꂽACe擾
	std::list<ARCHIVE_ENTRY_INFO_TREE*> items;
	GetSelectedItems(items);
	if(items.empty())return 0;
	ARCHIVE_ENTRY_INFO_TREE* lpNode=*(items.begin());

	if(lpNode->bDir){
		//Kw\𖳎ꍇɂ́A͖̓
		if(mr_Model.GetListMode()==FILELIST_TREE){
			mr_Model.MoveDownDir(lpNode);
		}
	}else{
		OpenAssociation(false,true);
	}
	return 0;
}


//J\On/Off؂ւ
//\:YJ\ɂAzl߂
//\:gĂȂɎwJǉ
void _ToggleColumn(int *lpArray,size_t size,FILEINFO_TYPE type)
{
	ASSERT(lpArray);
	if(!lpArray)return;

	for(size_t i=0;i<size;i++){
		if(type==lpArray[i]){
			//zl߂
			for(size_t j=i;j<size-1;j++){
				lpArray[j]=lpArray[j+1];
			}
			lpArray[size-1]=-1;
			return;
		}
		else if(-1==lpArray[i]){
			lpArray[i]=type;
			return;
		}
	}
}


//Jwb_/ENbN
LRESULT CFileListView::OnColumnRClick(int /*idCtrl*/, LPNMHDR pnmh, BOOL& bHandled)
{
	if(pnmh->hwndFrom!=GetHeader()){
		//bZ[W͏ȂƂɂ
		bHandled = FALSE;
		return 0;
	}

	//ENbNj[\
	POINT point;
	GetCursorPos(&point);
	CMenu cMenu;
	cMenu.LoadMenu(IDR_LISTVIEW_HEADER_MENU);
	CMenuHandle cSubMenu(cMenu.GetSubMenu(0));

	//--------------------------------
	// ej[ACe̗LE
	//--------------------------------

	int columnOrderArray[FILEINFO_ITEM_COUNT];
	int columnWidthArray[FILEINFO_ITEM_COUNT];
	GetColumnState(columnOrderArray, columnWidthArray);

	struct{
		FILEINFO_TYPE idx;
		UINT nMenuID;
	}menuTable[]={
		{FILEINFO_FULLPATH,			ID_MENUITEM_LISTVIEW_COLUMN_FULLPATH},
		{FILEINFO_ORIGINALSIZE,		ID_MENUITEM_LISTVIEW_COLUMN_ORIGINALSIZE},
		{FILEINFO_TYPENAME,			ID_MENUITEM_LISTVIEW_COLUMN_TYPENAME},
		{FILEINFO_FILETIME,			ID_MENUITEM_LISTVIEW_COLUMN_FILETIME},
		{FILEINFO_ATTRIBUTE,		ID_MENUITEM_LISTVIEW_COLUMN_ATTRIBUTE},
		{FILEINFO_COMPRESSEDSIZE,	ID_MENUITEM_LISTVIEW_COLUMN_COMPRESSEDSIZE},
		{FILEINFO_METHOD,			ID_MENUITEM_LISTVIEW_COLUMN_METHOD},
		{FILEINFO_RATIO,			ID_MENUITEM_LISTVIEW_COLUMN_RATIO},
		{FILEINFO_CRC,				ID_MENUITEM_LISTVIEW_COLUMN_CRC},
	};

	for(size_t i=0;i<COUNTOF(menuTable);i++){
		bool bEnabled=(-1!=UtilCheckNumberArray(columnOrderArray,COUNTOF(columnOrderArray),menuTable[i].idx));
		cSubMenu.CheckMenuItem(menuTable[i].nMenuID,MF_BYCOMMAND|(bEnabled?MF_CHECKED:MF_UNCHECKED));
	}

	//j[\:IR}hԂĂ
	int nRet=cSubMenu.TrackPopupMenu(TPM_NONOTIFY|TPM_RETURNCMD|TPM_LEFTALIGN|TPM_RIGHTBUTTON,point.x, point.y, m_hWnd,NULL);
	if(0==nRet){
		//Not Selected
		return 0;
	}else if(ID_MENUITEM_LISTVIEW_COLUMN_RESET==nRet){
		//
		for(size_t i=0;i<COUNTOF(columnOrderArray);i++){
			columnOrderArray[i]=i;
		}
	}else{
		for(size_t i=0;i<COUNTOF(menuTable);i++){
			if(menuTable[i].nMenuID==nRet){
				_ToggleColumn(columnOrderArray,COUNTOF(columnOrderArray),menuTable[i].idx);
			}
		}
	}

	SetColumnState(columnOrderArray, columnWidthArray);

	return 0;
}


//L[͂ɂt@C
LRESULT CFileListView::OnFindItem(LPNMHDR pnmh)
{
	ARCHIVE_ENTRY_INFO_TREE* lpCurrent=mr_Model.GetCurrentNode();
	ASSERT(lpCurrent);
	if(!lpCurrent)return -1;

	int iCount=lpCurrent->GetNumChildren();
	if(iCount<=0)return -1;

	LPNMLVFINDITEM lpFindInfo = (LPNMLVFINDITEM)pnmh;
	int iStart=lpFindInfo->iStart;
	if(iStart<0)iStart=0;
	if(lpFindInfo->lvfi.flags & LVFI_STRING || lpFindInfo->lvfi.flags & LVFI_PARTIAL){	//t@CŌ
		LPCTSTR lpFindString=lpFindInfo->lvfi.psz;
		size_t nLength=_tcslen(lpFindString);
		//OvŌ
		for(int i=iStart;i<iCount;i++){
			ARCHIVE_ENTRY_INFO_TREE* lpNode=mr_Model.GetFileListItemByIndex(i);
			ASSERT(lpNode);
			if(0==_tcsnicmp(lpFindString,lpNode->strTitle,nLength)){
				return i;
			}
		}
		if(lpFindInfo->lvfi.flags & LVFI_WRAP){
			for(int i=0;i<iStart;i++){
				ARCHIVE_ENTRY_INFO_TREE* lpNode=mr_Model.GetFileListItemByIndex(i);
				ASSERT(lpNode);
				if(0==_tcsnicmp(lpFindString,lpNode->strTitle,nLength)){
					return i;
				}
			}
		}
		return -1;
	}else{
		return -1;
	}
}



//\[g
LRESULT CFileListView::OnSortItem(LPNMHDR pnmh)
{
	LPNMLISTVIEW lpNMLV=(LPNMLISTVIEW)pnmh;
	int iCol=lpNMLV->iSubItem;
	SortItem(iCol);
	return 0;
}


void CFileListView::SortItem(int iCol)
{
	if(!(iCol >= 0 && iCol < FILEINFO_ITEM_COUNT))return;

	if(iCol==mr_Model.GetSortKeyType()){
		if(mr_Model.GetSortMode()){
			mr_Model.SetSortMode(false);
		}else{	//\[g
			mr_Model.SetSortKeyType(FILEINFO_INVALID);
			mr_Model.SetSortMode(true);
		}
	}else{
		mr_Model.SetSortKeyType(iCol);
		mr_Model.SetSortMode(true);
	}

	UpdateSortIcon();
}

void CFileListView::UpdateSortIcon()
{
	if(!IsWindow())return;
	//\[gԂɃJύX
	CHeaderCtrl hc=GetHeader();
	ASSERT(hc.IsWindow());
	if(hc.IsWindow()){
		int count=hc.GetItemCount();
		//ACR
		for(int i=0;i<count;i++){
			HDITEM hdi={0};
			hdi.mask=HDI_FORMAT;
			hc.GetItem(i,&hdi);
			if((hdi.fmt & HDF_IMAGE)){
				hdi.fmt&=~HDF_IMAGE;
				hc.SetItem(i,&hdi);
			}
		}

		//ACRZbg
		int iCol=mr_Model.GetSortKeyType();
		if(iCol!=FILEINFO_INVALID && iCol<count){
			HDITEM hdi={0};
			hdi.mask=HDI_FORMAT;

			hc.GetItem(iCol,&hdi);
			hdi.mask|=HDI_FORMAT|HDI_IMAGE;
			hdi.fmt|=HDF_IMAGE|HDF_BITMAP_ON_RIGHT;
			hdi.iImage=mr_Model.GetSortMode() ? 0 : 1;
			hc.SetItem(iCol,&hdi);
		}
	}
}

//JX^h[
DWORD CFileListView::OnPrePaint(int nID, LPNMCUSTOMDRAW lpnmcd)
{
	if(lpnmcd->hdr.hwndFrom == m_hWnd)
		return CDRF_NOTIFYITEMDRAW;
	else
		return CDRF_DODEFAULT;
}

DWORD CFileListView::OnItemPrePaint(int nID, LPNMCUSTOMDRAW lpnmcd)
{
	if(lpnmcd->hdr.hwndFrom == m_hWnd){
		LPNMLVCUSTOMDRAW lpnmlv = (LPNMLVCUSTOMDRAW)lpnmcd;

		ARCHIVE_ENTRY_INFO_TREE* lpNode=mr_Model.GetFileListItemByIndex(lpnmcd->dwItemSpec);
		if(lpNode){
			if(!lpNode->bSafe){
				//댯ȃA[JCuȂ̂ŐFt
				lpnmlv->clrText = RGB(255, 255, 255);
				lpnmlv->clrTextBk = RGB(255, 0, 0);
			}
		}
	}
	return CDRF_DODEFAULT;
}


//zXgr[̃ACe擾ɔ
LRESULT CFileListView::OnGetDispInfo(LPNMHDR pnmh)
{
	LV_DISPINFO* pstLVDInfo=(LV_DISPINFO*)pnmh;

	ARCHIVE_ENTRY_INFO_TREE* lpNode=mr_Model.GetFileListItemByIndex(pstLVDInfo->item.iItem);
	//ASSERT(lpNode);
	if(!lpNode)return 0;

	//Y`FbN
	ASSERT(pstLVDInfo->item.iSubItem>=0 && pstLVDInfo->item.iSubItem<FILEINFO_ITEM_COUNT);
	if(pstLVDInfo->item.iSubItem<0||pstLVDInfo->item.iSubItem>=FILEINFO_ITEM_COUNT)return 0;

	CString strBuffer;
	LPCTSTR lpText=NULL;
	switch(m_ColIdx2FileInfoType[pstLVDInfo->item.iSubItem]){
	case FILEINFO_FILENAME:	//t@C
		if(pstLVDInfo->item.mask & LVIF_TEXT)lpText=lpNode->strTitle;
		if(pstLVDInfo->item.mask & LVIF_IMAGE)pstLVDInfo->item.iImage=m_ShellDataManager.GetIconIndex(lpNode->strExt);
		break;
	case FILEINFO_FULLPATH:	//i[pX
		if(pstLVDInfo->item.mask & LVIF_TEXT){
			if(m_bPathOnly){
				//'\\'ł͂Ȃ'/'ŃpX؂Ătrue
				bool bSlash = (-1!=lpNode->strFullPath.Find(_T('/')));
				//xu
				strBuffer = lpNode->strFullPath;
				strBuffer.Replace(_T('/'),_T('\\'));

				//t@C
				TCHAR buf[_MAX_PATH+1];
				_tcsncpy_s(buf, strBuffer, COUNTOF(buf));
				PathRemoveBackslash(buf);
				PathRemoveFileSpec(buf);
				if(_tcslen(buf)==0){
					//ȂȂA܂胋[gfBNgɂ
					strBuffer = _T("\\");
				}else{
					PathAddBackslash(buf);
					strBuffer = buf;
				}

				if(bSlash){
					//ɖ߂u
					strBuffer.Replace(_T('\\'),_T('/'));
				}
				lpText = strBuffer;
			}else{
				lpText=lpNode->strFullPath;
			}
		}
		break;
	case FILEINFO_ORIGINALSIZE:	//TCY(kO)
		if(pstLVDInfo->item.mask & LVIF_TEXT){
			FormatFileSize(strBuffer,lpNode->llOriginalSize);
			lpText=strBuffer;
		}
		break;
	case FILEINFO_COMPRESSEDSIZE:	//TCY(k)
		if(pstLVDInfo->item.mask & LVIF_TEXT){
			FormatFileSize(strBuffer,lpNode->llCompressedSize);
			lpText=strBuffer;
		}
		break;
	case FILEINFO_TYPENAME:	//t@C^Cv
		if(pstLVDInfo->item.mask & LVIF_TEXT)lpText=m_ShellDataManager.GetTypeName(lpNode->strExt);
		break;
	case FILEINFO_FILETIME:	//t@C
		if(pstLVDInfo->item.mask & LVIF_TEXT){
			FormatFileTime(strBuffer,lpNode->cFileTime);
			lpText=strBuffer;
		}
		break;
	case FILEINFO_ATTRIBUTE:	//
		if(pstLVDInfo->item.mask & LVIF_TEXT){
			FormatAttribute(strBuffer,lpNode->nAttribute);
			lpText=strBuffer;
		}
		break;
	case FILEINFO_METHOD:	//k\bh
		if(pstLVDInfo->item.mask & LVIF_TEXT)lpText=lpNode->strMethod;
		break;
	case FILEINFO_RATIO:	//k
		if(pstLVDInfo->item.mask & LVIF_TEXT){
			FormatRatio(strBuffer,lpNode->wRatio);
			lpText=strBuffer;
		}
		break;
	case FILEINFO_CRC:	//CRC
		if(pstLVDInfo->item.mask & LVIF_TEXT){
			FormatCRC(strBuffer,lpNode->dwCRC);
			lpText=strBuffer;
		}
		break;
	}

	if(lpText){
		_tcsncpy_s(pstLVDInfo->item.pszText,pstLVDInfo->item.cchTextMax, lpText,pstLVDInfo->item.cchTextMax);
	}
	return 0;
}

//zXgr[̃c[`bv擾ɔ
LRESULT CFileListView::OnGetInfoTip(LPNMHDR pnmh)
{
	LPNMLVGETINFOTIP pGetInfoTip=(LPNMLVGETINFOTIP)pnmh;

	ARCHIVE_ENTRY_INFO_TREE* lpNode=mr_Model.GetFileListItemByIndex(pGetInfoTip->iItem);
	ASSERT(lpNode);
	if(!lpNode)return 0;
	CString strInfo;
	CString strBuffer;

	//t@C
	strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_FILENAME));
	strInfo+=_T(" : ");	strInfo+=lpNode->strTitle;		strInfo+=_T("\n");
	//i[pX
	strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_FULLPATH));
	strInfo+=_T(" : ");	strInfo+=lpNode->strFullPath;	strInfo+=_T("\n");
	//kOTCY
	FormatFileSize(strBuffer,lpNode->llOriginalSize);
	strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_ORIGINALSIZE));
	strInfo+=_T(" : ");	strInfo+=strBuffer;		strInfo+=_T("\n");
	//t@C^Cv
	strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_TYPENAME));
	strInfo+=_T(" : ");	strInfo+=m_ShellDataManager.GetTypeName(lpNode->strExt);	strInfo+=_T("\n");
	//t@C
	FormatFileTime(strBuffer,lpNode->cFileTime);
	strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_FILETIME));
	strInfo+=_T(" : ");	strInfo+=strBuffer;		strInfo+=_T("\n");
	//
	FormatAttribute(strBuffer,lpNode->nAttribute);
	strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_ATTRIBUTE));
	strInfo+=_T(" : ");	strInfo+=strBuffer;		strInfo+=_T("\n");
	//kTCY
	FormatFileSize(strBuffer,lpNode->llCompressedSize);
	strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_COMPRESSEDSIZE));
	strInfo+=_T(" : ");	strInfo+=strBuffer;		strInfo+=_T("\n");
	//k\bh
	strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_METHOD));
	strInfo+=_T(" : ");	strInfo+=lpNode->strMethod;	strInfo+=_T("\n");
	//k
	FormatRatio(strBuffer,lpNode->wRatio);
	strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_RATIO));
	strInfo+=_T(" : ");	strInfo+=strBuffer;		strInfo+=_T("\n");
	//CRC
	FormatCRC(strBuffer,lpNode->dwCRC);
	strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_COLUMN_CRC));
	strInfo+=_T(" : ");	strInfo+=strBuffer;		strInfo+=_T("\n");

	if(lpNode->nAttribute&FA_DIREC){
		//--------------
		// fBNg
		//--------------
		//qm[h̐\
		strInfo+=_T("\n");
		strInfo+=CString(MAKEINTRESOURCE(IDS_FILELIST_SUBITEM));
		strInfo+=_T(" : ");
		strInfo.AppendFormat(IDS_FILELIST_ITEMCOUNT,lpNode->GetNumChildren());
	}

	//InfoTipɐݒ
	_tcsncpy_s(pGetInfoTip->pszText,pGetInfoTip->cchTextMax,strInfo,pGetInfoTip->cchTextMax);
	return 0;
}

void CFileListView::FormatFileSizeInBytes(CString &Info,const LARGE_INTEGER &_Size)
{
	CString format(MAKEINTRESOURCE(IDS_ORDERUNIT_BYTE));

	std::wstringstream ss;
	ss.imbue(std::locale(""));
	ss << (unsigned __int64)_Size.QuadPart << std::endl;

	Info.Format(format,ss.str().c_str());
}

void CFileListView::FormatFileSize(CString &Info,const LARGE_INTEGER &_Size)
{
	LARGE_INTEGER Size=_Size;
	bool bInByte=m_bDisplayFileSizeInByte;
	Info.Empty();
	if(-1==Size.LowPart&&-1==Size.HighPart){
		Info=_T("---");
		return;
	}

	static CString OrderUnit[]={
		MAKEINTRESOURCE(IDS_ORDERUNIT_BYTE),
		MAKEINTRESOURCE(IDS_ORDERUNIT_KILOBYTE),
		MAKEINTRESOURCE(IDS_ORDERUNIT_MEGABYTE),
		MAKEINTRESOURCE(IDS_ORDERUNIT_GIGABYTE),
		MAKEINTRESOURCE(IDS_ORDERUNIT_TERABYTE),
	};	//TCY̒P
	static const int MAX_ORDERUNIT=COUNTOF(OrderUnit);

	if(bInByte){	//t@CTCYoCgPʂŕ\L
		FormatFileSizeInBytes(Info,_Size);
		return;
	}

	int Order=0;
	for(;Order<MAX_ORDERUNIT;Order++){
		if(Size.QuadPart<1024*1024){
			break;
		}
		Size.QuadPart=Int64ShrlMod32(Size.QuadPart,10);	//1024Ŋ
	}
	if(0==Order && Size.QuadPart<1024){
		//1KBɖȂ̂ŃoCgPʂł̂܂ܕ\L
		FormatFileSizeInBytes(Info,_Size);
	}else{
		TCHAR Buffer[64]={0};
		if(Order<MAX_ORDERUNIT-1){
			double SizeToDisplay=Size.QuadPart/1024.0;
			Order++;
			Info.Format(OrderUnit[Order],SizeToDisplay);
		}else{
			//ߑTCY
			Info.Format(OrderUnit[Order],Size.QuadPart);
		}
	}
}

void CFileListView::FormatFileTime(CString &Info,const FILETIME &rFileTime)
{
	Info.Empty();
	if(-1==rFileTime.dwHighDateTime && -1==rFileTime.dwLowDateTime){
		Info=_T("------");
	}else{
		FILETIME LocalFileTime;
		SYSTEMTIME SystemTime;

		FileTimeToLocalFileTime(&rFileTime,&LocalFileTime);
		FileTimeToSystemTime(&LocalFileTime, &SystemTime);
		TCHAR Buffer[64];

		wsprintf(Buffer,CString(MAKEINTRESOURCE(IDS_FILELIST_FILETIME_FORMAT)),
				SystemTime.wYear, SystemTime.wMonth, SystemTime.wDay,
				SystemTime.wHour, SystemTime.wMinute,SystemTime.wSecond);
		Info+=Buffer;
	}
}

void CFileListView::FormatAttribute(CString &strBuffer,int nAttribute)
{
	if(nAttribute&FA_UNKNOWN){
		strBuffer=_T("?????");
	}else{
		strBuffer="";
		strBuffer+=(nAttribute&FA_RDONLY)	? _T("R") : _T("-");
		strBuffer+=(nAttribute&FA_HIDDEN)	? _T("H") : _T("-");
		strBuffer+=(nAttribute&FA_SYSTEM)	? _T("S") : _T("-");
		strBuffer+=(nAttribute&FA_DIREC)	? _T("D") : _T("-");
		strBuffer+=(nAttribute&FA_ARCH)		? _T("A") : _T("-");
		strBuffer+=(nAttribute&FA_ENCRYPTED)? _T("P") : _T("-");
	}
}

void CFileListView::FormatRatio(CString &strBuffer,WORD wRatio)
{
	if(0xFFFF==wRatio)strBuffer=_T("?????");	//擾s
	else strBuffer.Format(_T("%.1f%%"),(double)wRatio/10.0);
}

void CFileListView::FormatCRC(CString &strBuffer,DWORD dwCRC)
{
	if(-1==dwCRC)strBuffer=_T("?????");	//擾s
	else strBuffer.Format(_T("%08x"),dwCRC);
}

//-------

void CFileListView::OnClearTemporary(UINT uNotifyCode,int nID,HWND hWndCtrl)
{
	mr_Model.ClearTempDir();
}


void CFileListView::OnOpenAssociation(UINT uNotifyCode,int nID,HWND hWndCtrl)
{
	//trueȂ瑶݂e|t@C폜Ă𓀂
	const bool bOverwrite=(nID==ID_MENUITEM_OPEN_ASSOCIATION_OVERWRITE);
	OpenAssociation(bOverwrite,true);
}


void CFileListView::OnExtractTemporary(UINT uNotifyCode,int nID,HWND hWndCtrl)
{
	//㏑͂邪AJȂ
	OpenAssociation(true,false);
}

//bOverwrite:trueȂ瑶݂e|t@C폜Ă𓀂
bool CFileListView::OpenAssociation(bool bOverwrite,bool bOpen)
{
	if(!mr_Model.IsExtractEachSupported()){
		//It@C̉𓀂̓T|[gĂȂ
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_FILELIST_EXTRACT_SELECTED_NOT_SUPPORTED)));
		return false;
	}

	if(!mr_Model.CheckArchiveExists()){	//݂ȂȂG[
		CString msg;
		msg.Format(IDS_ERROR_FILE_NOT_FOUND,mr_Model.GetArchiveFileName());
		ErrorMessage(msg);
		return false;
	}

	//IꂽACe
	std::list<ARCHIVE_ENTRY_INFO_TREE*> items;
	GetSelectedItems(items);

	if(!items.empty()){
		std::list<CString> filesList;
		CString strLog;
		if(!mr_Model.MakeSureItemsExtracted(NULL,mr_Model.GetRootNode(),items,filesList,bOverwrite,strLog)){
			CLogDialog LogDialog;
			LogDialog.SetData(strLog);
			LogDialog.DoModal();
		}
		if(bOpen)OpenAssociation(filesList);
	}

	return true;
}

void CFileListView::OpenAssociation(const std::list<CString> &filesList)
{
	for(std::list<CString>::const_iterator ite=filesList.begin();ite!=filesList.end();++ite){
		//ۂꂽ㏑ǉ𓀂Ȃ;fBNgȂ狑ۂ̂݃`FbN
		bool bDenyOnly=BOOL2bool(::PathIsDirectory(*ite));//lpNode->bDir;
		if(UtilPathAcceptSpec(*ite,mr_Model.GetOpenAssocExtDeny(),mr_Model.GetOpenAssocExtAccept(),bDenyOnly)){
			::ShellExecute(GetDesktopWindow(),NULL,*ite,NULL,NULL,SW_SHOW);
			TRACE(_T("%s\n"),(LPCTSTR)*ite);
			//::ShellExecute(GetDesktopWindow(),_T("explore"),*ite,NULL,NULL,SW_SHOW);
		}else{
			::MessageBeep(MB_ICONEXCLAMATION);
		}
	}
}

HRESULT CFileListView::AddItems(const std::list<CString> &fileList,LPCTSTR strDest)
{
	//ǉJn
	::EnableWindow(m_hFrameWnd,FALSE);
	CString strLog;

	HRESULT hr=mr_Model.AddItem(fileList,strDest,strLog);

	::EnableWindow(m_hFrameWnd,TRUE);
	::SetForegroundWindow(m_hFrameWnd);

	if(FAILED(hr) || S_FALSE==hr){
		CString msg;
		switch(hr){
		case E_LF_SAME_INPUT_AND_OUTPUT:	//A[JCugǉ悤Ƃ
			msg.Format(IDS_ERROR_SAME_INPUT_AND_OUTPUT,mr_Model.GetArchiveFileName());
			ErrorMessage(msg);
			break;
		case E_LF_UNICODE_NOT_SUPPORTED:	//t@CUNICODEt@Ck悤Ƃ
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_UNICODEPATH)));
			break;
		case S_FALSE:	//ǉɖ
			{
				CLogDialog LogDialog;
				LogDialog.SetData(strLog);
				LogDialog.DoModal();
			}
			break;
		default:
			ASSERT(!"This code cannot be run");
		}
		return E_INVALIDARG;
	}
	//A[JCuXV
	::PostMessage(m_hFrameWnd,WM_FILELIST_REFRESH,0,0);
	return S_OK;
}

void CFileListView::OnAddItems(UINT uNotifyCode,int nID,HWND hWndCtrl)
{
	ASSERT(mr_Model.IsAddItemsSupported());
	if(!mr_Model.IsAddItemsSupported())return;

	CString strDest;	//荞ސ
	//JgtH_ɒǉ
	ArcEntryInfoTree_GetNodePathRelative(mr_Model.GetCurrentNode(),mr_Model.GetRootNode(),strDest);

	std::list<CString> fileList;
	if(nID==ID_MENUITEM_ADD_FILE){		//t@Cǉ
		//uSẴt@CṽtB^
		CString strAnyFile(MAKEINTRESOURCE(IDS_FILTER_ANYFILE));
		std::vector<TCHAR> filter(strAnyFile.GetLength()+1);
		UtilMakeFilterString(strAnyFile,&filter[0],filter.size());

		CMultiFileDialog dlg(NULL, NULL, OFN_NOCHANGEDIR|OFN_DONTADDTORECENT|OFN_PATHMUSTEXIST|OFN_HIDEREADONLY|OFN_ALLOWMULTISELECT,&filter[0]);
		if(IDOK==dlg.DoModal()){
			//t@Co
			CString tmp;
			if(dlg.GetFirstPathName(tmp)){
				do{
					fileList.push_back(tmp);
				}while(dlg.GetNextPathName(tmp));
			}
		}
	}else{		//tH_ǉ
		CLFFolderDialog dlg(m_hFrameWnd,NULL,BIF_RETURNONLYFSDIRS|BIF_NEWDIALOGSTYLE);
		if(IDOK==dlg.DoModal()){
			fileList.push_back(dlg.GetFolderPath());
		}
	}

	if(!fileList.empty()){
		//ǉJn
		AddItems(fileList,strDest);
	}
}


void CFileListView::EnableDropTarget(bool bEnable)
{
	if(bEnable){
		//hbv󂯓ݒ
		::RegisterDragDrop(m_hWnd,&m_DropTarget);
	}else{
		//hbv󂯓Ȃ
		::RevokeDragDrop(m_hWnd);
	}
}


//---------------------------------------------------------
//    IDropCommunicator̎:hbO&hbvɂ鈳k
//---------------------------------------------------------
HRESULT CFileListView::DragEnter(IDataObject *lpDataObject,POINTL &pt,DWORD &dwEffect)
{
	return DragOver(lpDataObject,pt,dwEffect);
}

HRESULT CFileListView::DragLeave()
{
	//SẴnCCg𖳌
	SetItemState( -1, ~LVIS_DROPHILITED, LVIS_DROPHILITED);
	m_nDropHilight=-1;
	return S_OK;
}

HRESULT CFileListView::DragOver(IDataObject *,POINTL &pt,DWORD &dwEffect)
{
	//tH[}bgɑΉ
	if(!m_DropTarget.QueryFormat(CF_HDROP) || !mr_Model.IsAddItemsSupported()){	//t@Cp
		//t@Cł͂Ȃ̂ŋ
		dwEffect = DROPEFFECT_NONE;
	}else{
		//---hbvACe擾
		CPoint ptTemp(pt.x,pt.y);
		ScreenToClient(&ptTemp);
		int nIndex=HitTest(ptTemp,NULL);

		//---邽߁AOƓACenCCgȂnCCgNAȂ
		if(nIndex!=m_nDropHilight){
			//SẴnCCg𖳌
			SetItemState( -1, ~LVIS_DROPHILITED, LVIS_DROPHILITED);
			m_nDropHilight=-1;
			ARCHIVE_ENTRY_INFO_TREE* lpNode=mr_Model.GetFileListItemByIndex(nIndex);
			if(lpNode){		//ACeDnD
				//ACetH_nCCg
				if(lpNode->bDir){
					SetItemState( nIndex, LVIS_DROPHILITED, LVIS_DROPHILITED);
					m_nDropHilight=nIndex;
				}
			}
		}
		dwEffect = DROPEFFECT_COPY;
	}
	return S_OK;
}

//t@C̃hbv
HRESULT CFileListView::Drop(IDataObject *lpDataObject,POINTL &pt,DWORD &dwEffect)
{
	//SẴnCCg𖳌
	SetItemState( -1, ~LVIS_DROPHILITED, LVIS_DROPHILITED);
	m_nDropHilight=-1;

	//t@C擾
	std::list<CString> fileList;
	if(S_OK==m_DropTarget.GetDroppedFiles(lpDataObject,fileList)){
		dwEffect = DROPEFFECT_COPY;

		//---hbv
		CPoint ptTemp(pt.x,pt.y);
		ScreenToClient(&ptTemp);
		int nIndex=HitTest(ptTemp,NULL);

		CString strDest;	//荞ސ
		ARCHIVE_ENTRY_INFO_TREE* lpNode=mr_Model.GetFileListItemByIndex(nIndex);
		if(lpNode){		//ACeDnD
			//ACetH_炻̃tH_ɒǉ
			if(lpNode->bDir){
				ArcEntryInfoTree_GetNodePathRelative(lpNode,mr_Model.GetRootNode(),strDest);
			}else{
				//JgtH_ɒǉ
				ArcEntryInfoTree_GetNodePathRelative(mr_Model.GetCurrentNode(),mr_Model.GetRootNode(),strDest);
			}
		}else{	//ACeODnD->JgtH_ɒǉ
			ArcEntryInfoTree_GetNodePathRelative(mr_Model.GetCurrentNode(),mr_Model.GetRootNode(),strDest);
		}
		TRACE(_T("Target:%s\n"),(LPCTSTR)strDest);

		//ǉJn
		return AddItems(fileList,strDest);
	}else{
		//󂯓łȂ`
		dwEffect = DROPEFFECT_NONE;
		return S_FALSE;	//S_OK
	}
}

//-----------------------------
// hbO&hbvɂ
//-----------------------------

LRESULT CFileListView::OnBeginDrag(LPNMHDR pnmh)
{
	if(!mr_Model.IsExtractEachSupported()){
		//It@C̉𓀂̓T|[gĂȂ
		//ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_FILELIST_EXTRACT_SELECTED_NOT_SUPPORTED)));
		MessageBeep(MB_ICONASTERISK);
		return 0;
	}

	if(!mr_Model.CheckArchiveExists()){	//݂ȂȂG[
		return 0;
	}
	//IꂽACe
	std::list<ARCHIVE_ENTRY_INFO_TREE*> items;
	GetSelectedItems(items);
	if(items.empty()){	//{蓾Ȃ
		ASSERT(!"This code cannot be run");
		return 0;
	}

	if(!m_TempDirMgr.ClearSubDir()){
		//e|fBNgɏoȂ
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_CANT_CLEAR_TEMPDIR)));
		return 0;
	}else{
		::EnableWindow(m_hFrameWnd,FALSE);

		//hbO&hbvŉ
		CString strLog;
		HRESULT hr=m_DnDSource.DragDrop(mr_Model,items,mr_Model.GetCurrentNode(),m_TempDirMgr.GetDirPath(),strLog);
		if(FAILED(hr)){
			if(hr==E_ABORT){
				CLogDialog LogDialog;
				LogDialog.SetData(strLog);
				LogDialog.DoModal();
			}else{
				ErrorMessage(strLog);
			}
		}

		::EnableWindow(m_hFrameWnd,TRUE);
		::SetForegroundWindow(m_hFrameWnd);
	}
	return 0;
}

void CFileListView::OnSelectAll(UINT,int,HWND)
{
	SetItemState(-1, LVIS_SELECTED, LVIS_SELECTED);
	SetFocus();
}


void CFileListView::OnDelete(UINT uNotifyCode,int nID,HWND hWndCtrl)
{
	if(!mr_Model.IsDeleteItemsSupported()){
		//It@C̍폜̓T|[gĂȂ
		if(1==uNotifyCode){	//ANZ[^瑀
			MessageBeep(MB_OK);
		}else{
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_FILELIST_DELETE_SELECTED_NOT_SUPPORTED)));
		}
		return;// false;
	}

	//Iꂽt@C
	std::list<ARCHIVE_ENTRY_INFO_TREE*> items;
	GetSelectedItems(items);

	//t@CIĂȂ΃G[
	ASSERT(!items.empty());
	if(items.empty()){
		return;// false;
	}

	//mF
	if(IDYES!=MessageBox(CString(MAKEINTRESOURCE(IDS_ASK_FILELIST_DELETE_SELECTED)),UtilGetMessageCaption(),MB_YESNO|MB_DEFBUTTON2|MB_ICONEXCLAMATION)){
		return;
	}

	//----------------
	// t@C
	//----------------
	//EBhEgps
	::EnableWindow(m_hFrameWnd,FALSE);
	CString strLog;
	bool bRet=mr_Model.DeleteItems(items,strLog);
	//EBhEgp\
	::EnableWindow(m_hFrameWnd,TRUE);
	SetForegroundWindow(m_hFrameWnd);
	if(!bRet){
		CLogDialog LogDlg;
		LogDlg.SetData(strLog);
		LogDlg.DoModal(m_hFrameWnd);
	}

	//A[JCueXV
	::PostMessage(m_hFrameWnd,WM_FILELIST_REFRESH,NULL,NULL);
}

//ReLXgj[J
void CFileListView::OnContextMenu(HWND hWndCtrl,CPoint &Point)
{
	//IĂȂΕ\Ȃ
	if(GetSelectedCount()==0)return;

	if(-1==Point.x&&-1==Point.y){
		//L[{[h̓͂łꍇ
		//Xgr[̍ɕ\
		Point.x=Point.y=0;
		ClientToScreen(&Point);
	}else{
		//}EXE{^t@CꗗEBhE*J*ŉꂽ̂łΕԂ
		HDHITTESTINFO hdHitTestInfo={0};
		hdHitTestInfo.pt=Point;
		GetHeader().ScreenToClient(&hdHitTestInfo.pt);
		GetHeader().HitTest(&hdHitTestInfo);
		if(hdHitTestInfo.flags&HHT_ONHEADER){
			return;
		}
	}

	//𓀂gpłȂȂAj[\ӖȂ
	//TODO:폜j[͂ǂ?𓀂łɍ폜\͂قڂ蓾Ȃ
	if(!mr_Model.IsExtractEachSupported()){
		MessageBeep(MB_ICONASTERISK);
		return;
	}

	//---ENbNj[\
	CMenu cMenu;
	cMenu.LoadMenu(IDR_FILELIST_POPUP);
	CMenuHandle cSubMenu(cMenu.GetSubMenu(0));

	//R}hǉ邽߂̃Tuj[T
	int MenuCount=cSubMenu.GetMenuItemCount();
	int iIndex=-1;
	for(int i=0;i<=MenuCount;i++){
		if(-1==cSubMenu.GetMenuItemID(i)){	//|bvAbv̐e
			iIndex=i;
			break;
		}
	}
	ASSERT(-1!=iIndex);
	if(-1!=iIndex){
		MenuCommand_MakeUserAppMenu(cSubMenu.GetSubMenu(iIndex));
		MenuCommand_MakeSendToMenu(cSubMenu.GetSubMenu(iIndex+1));
	}

	cSubMenu.TrackPopupMenu(TPM_LEFTALIGN|TPM_RIGHTBUTTON,Point.x, Point.y, m_hWnd,NULL);
}

void CFileListView::OnOpenWithUserApp(UINT uNotifyCode,int nID,HWND hWndCtrl)
{
	if(nID<ID_MENUITEM_USERAPP_END){
		//LhaForgeݒ̃R}h
		OnUserApp(MenuCommand_GetCmdArray(),nID-ID_MENUITEM_USERAPP_BEGIN);
	}else{
		//SendTõR}h
		OnSendToApp(nID-ID_MENUITEM_USERAPP_END);
	}
}


bool CFileListView::OnUserApp(const std::vector<CMenuCommandItem> &menuCommandArray,UINT nID)	//uvOŊJṽnh
{
	ASSERT(!menuCommandArray.empty());
	ASSERT(nID<menuCommandArray.size());
	if(nID>=menuCommandArray.size())return false;

	if(!mr_Model.IsOK())return false;

	//IꂽACe
	std::list<ARCHIVE_ENTRY_INFO_TREE*> items;
	GetSelectedItems(items);

	std::list<CString> filesList;
	if(!items.empty()){
		CString strLog;
		if(!mr_Model.MakeSureItemsExtracted(NULL,mr_Model.GetRootNode(),items,filesList,false,strLog)){
			CLogDialog LogDialog;
			LogDialog.SetData(strLog);
			LogDialog.DoModal();
			return false;
		}
	}

	//---s擾
	//p[^WJɕKvȏ
	std::map<stdString,CString> envInfo;
	UtilMakeExpandInformation(envInfo);

	//R}hEp[^WJ
	CString strCmd,strParam,strDir;
	UtilExpandTemplateString(strCmd,  menuCommandArray[nID].Path, envInfo);	//R}h
	UtilExpandTemplateString(strParam,menuCommandArray[nID].Param,envInfo);	//p[^
	UtilExpandTemplateString(strDir,  menuCommandArray[nID].Dir,  envInfo);	//fBNg

	//u
	if(-1!=strParam.Find(_T("%F"))){
		//t@CꗗAč쐬
		CString strFileList;
		for(std::list<CString>::iterator ite=filesList.begin();ite!=filesList.end();++ite){
			CPath path=*ite;
			path.QuoteSpaces();
			strFileList+=(LPCTSTR)path;
			strFileList+=_T(" ");
		}
		strParam.Replace(_T("%F"),strFileList);
		//---s
		::ShellExecute(GetDesktopWindow(),NULL,strCmd,strParam,strDir,SW_SHOW);
	}else if(-1!=strParam.Find(_T("%S"))){
		for(std::list<CString>::iterator ite=filesList.begin();ite!=filesList.end();++ite){
			CPath path=*ite;
			path.QuoteSpaces();

			CString strParamTmp=strParam;
			strParamTmp.Replace(_T("%S"),(LPCTSTR)path);
			//---s
			::ShellExecute(GetDesktopWindow(),NULL,strCmd,strParamTmp,strDir,SW_SHOW);
		}
	}else{
		::ShellExecute(GetDesktopWindow(),NULL,strCmd,strParam,strDir,SW_SHOW);
	}

	return true;
}

bool CFileListView::OnSendToApp(UINT nID)	//uvOŊJṽnh
{
	ASSERT(MenuCommand_GetNumSendToCmd());
	ASSERT(nID<MenuCommand_GetNumSendToCmd());
	if(nID>=MenuCommand_GetNumSendToCmd())return false;

	if(!mr_Model.IsOK())return false;

	//---I𓀊Jn
	//IꂽACe
	std::list<ARCHIVE_ENTRY_INFO_TREE*> items;
	GetSelectedItems(items);

	std::list<CString> filesList;
	if(!items.empty()){
		CString strLog;
		if(!mr_Model.MakeSureItemsExtracted(NULL,mr_Model.GetRootNode(),items,filesList,false,strLog)){
			CLogDialog LogDialog;
			LogDialog.SetData(strLog);
			LogDialog.DoModal();
			return false;
		}
	}

	//u
	const std::vector<SHORTCUTINFO>& sendToCmd=MenuCommand_GetSendToCmdArray();
	if(PathIsDirectory(sendToCmd[nID].strCmd)){
		//Ώۂ̓fBNgȂ̂ŁARs[
		CString strFiles;
		for(std::list<CString>::const_iterator ite=filesList.begin();ite!=filesList.end();++ite){
			CPath file=*ite;
			file.RemoveBackslash();
			strFiles+=file;
			strFiles+=_T('|');
		}
		strFiles+=_T('|');
		//TRACE(strFiles);
		std::vector<TCHAR> srcBuf(strFiles.GetLength()+1);
		UtilMakeFilterString(strFiles,&srcBuf[0],srcBuf.size());

		CPath destDir=sendToCmd[nID].strCmd;
		destDir.AddBackslash();
		//WindowsW̃Rs[
		SHFILEOPSTRUCT fileOp={0};
		fileOp.wFunc=FO_COPY;
		fileOp.fFlags=FOF_NOCONFIRMMKDIR|FOF_NOCOPYSECURITYATTRIBS|FOF_NO_CONNECTED_ELEMENTS;
		fileOp.pFrom=&srcBuf[0];
		fileOp.pTo=destDir;

		//Rs[s
		if(::SHFileOperation(&fileOp)){
			//G[
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_FILE_COPY)));
			return false;
		}else if(fileOp.fAnyOperationsAborted){
			//LZ
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_USERCANCEL)));
			return false;
		}
		return true;
	}else{
		//̓vO
		//t@CꗗAč쐬
		CString strFileList;
		for(std::list<CString>::iterator ite=filesList.begin();ite!=filesList.end();++ite){
			CPath path=*ite;
			path.QuoteSpaces();
			strFileList+=(LPCTSTR)path;
			strFileList+=_T(" ");
		}
		CString strParam=sendToCmd[nID].strParam+_T(" ")+strFileList;
		//---s
		CPath cmd=sendToCmd[nID].strCmd;
		cmd.QuoteSpaces();
		CPath workDir=sendToCmd[nID].strWorkingDir;
		workDir.QuoteSpaces();
		::ShellExecute(GetDesktopWindow(),NULL,cmd,strParam,workDir,SW_SHOW);
	}

	return true;
}

void CFileListView::OnExtractItem(UINT,int nID,HWND)
{
	//A[JCuƓtH_ɉ𓀂ꍇtrue
	const bool bSameDir=(ID_MENUITEM_EXTRACT_SELECTED_SAMEDIR==nID);

	if(!mr_Model.IsOK()){
		return;// false;
	}
	if(!mr_Model.IsExtractEachSupported()){
		//It@C̉𓀂̓T|[gĂȂ
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_FILELIST_EXTRACT_SELECTED_NOT_SUPPORTED)));
		return;// false;
	}

	//IꂽACe
	std::list<ARCHIVE_ENTRY_INFO_TREE*> items;
	GetSelectedItems(items);
	if(items.empty()){
		//Iꂽt@CȂ
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_FILELIST_NOT_SELECTED)));
		return;// false;
	}

	//
	CString strLog;
	HRESULT hr=mr_Model.ExtractItems(m_hFrameWnd,bSameDir,items,mr_Model.GetCurrentNode(),strLog);

	SetForegroundWindow(m_hFrameWnd);

	if(FAILED(hr)){
		CLogDialog LogDialog;
		LogDialog.SetData(strLog);
		LogDialog.DoModal();
	}
}

void CFileListView::OnFindItem(UINT uNotifyCode,int nID,HWND hWndCtrl)
{
	if(ID_MENUITEM_FINDITEM_END==nID){
		mr_Model.EndFindItem();
	}else{
		CString strSpec;
		if(UtilInputText(CString(MAKEINTRESOURCE(IDS_INPUT_FIND_PARAM)),strSpec)){
			mr_Model.EndFindItem();
			ARCHIVE_ENTRY_INFO_TREE* lpFound=mr_Model.FindItem(strSpec,mr_Model.GetCurrentNode());
			mr_Model.SetCurrentNode(lpFound);
		}
	}
}

void CFileListView::OnShowCustomizeColumn(UINT,int,HWND)
{
	//Jwb_ҏWj[\邽߁AJwb_̉ENbNG~[g
	BOOL bTemp;
	NMHDR nmhdr;
	nmhdr.hwndFrom=GetHeader();
	OnColumnRClick(0, &nmhdr, bTemp);
}
